@@ -47,7 +47,6 @@ http://github.com/memcached/memcached/blob/1.4.2/doc/protocol.txt
 import six.moves.cPickle as pickle
 import json
 import logging
-import re
 import time
 from bisect import bisect
 from swift import gettext_ as _
@@ -57,7 +56,7 @@ from eventlet.green import socket
 from eventlet.pools import Pool
 from eventlet import Timeout
 from six.moves import range
-
+from swift.common import utils
 
 DEFAULT_MEMCACHED_PORT = 11211
 
@@ -106,48 +105,18 @@ class MemcacheConnPool(Pool):
     Connection pool for Memcache Connections
 
     The *server* parameter can be a hostname, an IPv4 address, or an IPv6
-    address with an optional port. If an IPv6 address is specified it **must**
-    be enclosed in [], like *[::1]* or *[::1]:11211*. This follows the accepted
-    prescription for `IPv6 host literals`_.
-
-    Examples::
-
-        memcache.local:11211
-        127.0.0.1:11211
-        [::1]:11211
-        [::1]
-
-    .. _IPv6 host literals: https://tools.ietf.org/html/rfc3986#section-3.2.2
+    address with an optional port. See
+    :func:`swift.common.utils.parse_socket_string` for details.
     """
-    IPV6_RE = re.compile("^\[(?P<address>.*)\](:(?P<port>[0-9]+))?$")
 
     def __init__(self, server, size, connect_timeout):
         Pool.__init__(self, max_size=size)
-        self.server = server
+        self.host, self.port = utils.parse_socket_string(
+            server, DEFAULT_MEMCACHED_PORT)
         self._connect_timeout = connect_timeout
 
-    def _get_addr(self):
-        port = DEFAULT_MEMCACHED_PORT
-        # IPv6 addresses must be between '[]'
-        if self.server.startswith('['):
-            match = MemcacheConnPool.IPV6_RE.match(self.server)
-            if not match:
-                raise ValueError("Invalid IPv6 address: %s" % self.server)
-            host = match.group('address')
-            port = match.group('port') or port
-        else:
-            if ':' in self.server:
-                tokens = self.server.split(':')
-                if len(tokens) > 2:
-                    raise ValueError("IPv6 addresses must be between '[]'")
-                host, port = tokens
-            else:
-                host = self.server
-        return (host, port)
-
     def create(self):
-        host, port = self._get_addr()
-        addrs = socket.getaddrinfo(host, port, socket.AF_UNSPEC,
+        addrs = socket.getaddrinfo(self.host, self.port, socket.AF_UNSPEC,
                                    socket.SOCK_STREAM)
         family, socktype, proto, canonname, sockaddr = addrs[0]
         sock = socket.socket(family, socket.SOCK_STREAM)
