@@ -14,7 +14,8 @@
 # limitations under the License.
 
 import array
-import cPickle as pickle
+import six.moves.cPickle as pickle
+import json
 from collections import defaultdict
 from gzip import GzipFile
 from os.path import getmtime
@@ -26,7 +27,9 @@ from hashlib import md5
 from itertools import chain
 from tempfile import NamedTemporaryFile
 
-from swift.common.utils import hash_path, validate_configuration, json
+from six.moves import range
+
+from swift.common.utils import hash_path, validate_configuration
 from swift.common.ring.utils import tiers_for_dev
 
 
@@ -43,22 +46,42 @@ class RingData(object):
                 dev.setdefault("region", 1)
 
     @classmethod
-    def deserialize_v1(cls, gz_file):
+    def deserialize_v1(cls, gz_file, metadata_only=False):
+        """
+        Deserialize a v1 ring file into a dictionary with `devs`, `part_shift`,
+        and `replica2part2dev_id` keys.
+
+        If the optional kwarg `metadata_only` is True, then the
+        `replica2part2dev_id` is not loaded and that key in the returned
+        dictionary just has the value `[]`.
+
+        :param file gz_file: An opened file-like object which has already
+                             consumed the 6 bytes of magic and version.
+        :param bool metadata_only: If True, only load `devs` and `part_shift`
+        :returns: A dict containing `devs`, `part_shift`, and
+                  `replica2part2dev_id`
+        """
+
         json_len, = struct.unpack('!I', gz_file.read(4))
         ring_dict = json.loads(gz_file.read(json_len))
         ring_dict['replica2part2dev_id'] = []
+
+        if metadata_only:
+            return ring_dict
+
         partition_count = 1 << (32 - ring_dict['part_shift'])
-        for x in xrange(ring_dict['replica_count']):
+        for x in range(ring_dict['replica_count']):
             ring_dict['replica2part2dev_id'].append(
                 array.array('H', gz_file.read(2 * partition_count)))
         return ring_dict
 
     @classmethod
-    def load(cls, filename):
+    def load(cls, filename, metadata_only=False):
         """
         Load ring data from a file.
 
         :param filename: Path to a file serialized by the save() method.
+        :param bool metadata_only: If True, only load `devs` and `part_shift`.
         :returns: A RingData instance containing the loaded data.
         """
         gz_file = GzipFile(filename, 'rb')
@@ -69,15 +92,18 @@ class RingData(object):
         # See if the file is in the new format
         magic = gz_file.read(4)
         if magic == 'R1NG':
-            version, = struct.unpack('!H', gz_file.read(2))
-            if version == 1:
-                ring_data = cls.deserialize_v1(gz_file)
+            format_version, = struct.unpack('!H', gz_file.read(2))
+            if format_version == 1:
+                ring_data = cls.deserialize_v1(
+                    gz_file, metadata_only=metadata_only)
             else:
-                raise Exception('Unknown ring format version %d' % version)
+                raise Exception('Unknown ring format version %d' %
+                                format_version)
         else:
             # Assume old-style pickled ring
             gz_file.seek(0)
             ring_data = pickle.load(gz_file)
+
         if not hasattr(ring_data, 'devs'):
             ring_data = RingData(ring_data['replica2part2dev_id'],
                                  ring_data['devs'], ring_data['part_shift'])
@@ -97,29 +123,25 @@ class RingData(object):
         for part2dev_id in ring['replica2part2dev_id']:
             file_obj.write(part2dev_id.tostring())
 
-    def save(self, filename):
+    def save(self, filename, mtime=1300507380.0):
         """
         Serialize this RingData instance to disk.
 
         :param filename: File into which this instance should be serialized.
+        :param mtime: time used to override mtime for gzip, default or None
+                      if the caller wants to include time
         """
         # Override the timestamp so that the same ring data creates
         # the same bytes on disk. This makes a checksum comparison a
         # good way to see if two rings are identical.
-        #
-        # This only works on Python 2.7; on 2.6, we always get the
-        # current time in the gzip output.
         tempf = NamedTemporaryFile(dir=".", prefix=filename, delete=False)
-        try:
-            gz_file = GzipFile(filename, mode='wb', fileobj=tempf,
-                               mtime=1300507380.0)
-        except TypeError:
-            gz_file = GzipFile(filename, mode='wb', fileobj=tempf)
+        gz_file = GzipFile(filename, mode='wb', fileobj=tempf, mtime=mtime)
         self.serialize_v1(gz_file)
         gz_file.close()
         tempf.flush()
         os.fsync(tempf.fileno())
         tempf.close()
+        os.chmod(tempf.name, 0o644)
         os.rename(tempf.name, filename)
 
     def to_dict(self):
@@ -171,22 +193,32 @@ class Ring(object):
             self._part_shift = ring_data._part_shift
             self._rebuild_tier_data()
 
-            # Do this now, when we know the data has changed, rather then
+            # Do this now, when we know the data has changed, rather than
             # doing it on every call to get_more_nodes().
+            #
+            # Since this is to speed up the finding of handoffs, we only
+            # consider devices with at least one partition assigned. This
+            # way, a region, zone, or server with no partitions assigned
+            # does not count toward our totals, thereby keeping the early
+            # bailouts in get_more_nodes() working.
+            dev_ids_with_parts = set()
+            for part2dev_id in self._replica2part2dev_id:
+                for dev_id in part2dev_id:
+                    dev_ids_with_parts.add(dev_id)
+
             regions = set()
             zones = set()
-            ip_ports = set()
+            ips = set()
             self._num_devs = 0
             for dev in self._devs:
-                if dev:
+                if dev and dev['id'] in dev_ids_with_parts:
                     regions.add(dev['region'])
                     zones.add((dev['region'], dev['zone']))
-                    ip_ports.add((dev['region'], dev['zone'],
-                                  dev['ip'], dev['port']))
+                    ips.add((dev['region'], dev['zone'], dev['ip']))
                     self._num_devs += 1
             self._num_regions = len(regions)
             self._num_zones = len(zones)
-            self._num_ip_ports = len(ip_ports)
+            self._num_ips = len(ips)
 
     def _rebuild_tier_data(self):
         self.tier2devs = defaultdict(list)
@@ -238,8 +270,8 @@ class Ring(object):
                 dev_id = r2p2d[part]
                 if dev_id not in seen_ids:
                     part_nodes.append(self.devs[dev_id])
-                seen_ids.add(dev_id)
-        return part_nodes
+                    seen_ids.add(dev_id)
+        return [dict(node, index=i) for i, node in enumerate(part_nodes)]
 
     def get_part(self, account, container=None, obj=None):
         """
@@ -287,6 +319,7 @@ class Ring(object):
 
         ======  ===============================================================
         id      unique integer identifier amongst devices
+        index   offset into the primary node list for the partition
         weight  a float of the relative weight of this device as compared to
                 others; this indicates how many partitions the builder will try
                 to assign to this device
@@ -324,8 +357,8 @@ class Ring(object):
         used = set(d['id'] for d in primary_nodes)
         same_regions = set(d['region'] for d in primary_nodes)
         same_zones = set((d['region'], d['zone']) for d in primary_nodes)
-        same_ip_ports = set((d['region'], d['zone'], d['ip'], d['port'])
-                            for d in primary_nodes)
+        same_ips = set(
+            (d['region'], d['zone'], d['ip']) for d in primary_nodes)
 
         parts = len(self._replica2part2dev_id[0])
         start = struct.unpack_from(
@@ -334,9 +367,9 @@ class Ring(object):
         # Multiple loops for execution speed; the checks and bookkeeping get
         # simpler as you go along
         hit_all_regions = len(same_regions) == self._num_regions
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
             if hit_all_regions:
                 # At this point, there are no regions left untouched, so we
                 # can stop looking.
@@ -351,17 +384,17 @@ class Ring(object):
                         used.add(dev_id)
                         same_regions.add(region)
                         zone = dev['zone']
-                        ip_port = (region, zone, dev['ip'], dev['port'])
+                        ip = (region, zone, dev['ip'])
                         same_zones.add((region, zone))
-                        same_ip_ports.add(ip_port)
+                        same_ips.add(ip)
                         if len(same_regions) == self._num_regions:
                             hit_all_regions = True
                             break
 
         hit_all_zones = len(same_zones) == self._num_zones
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
             if hit_all_zones:
                 # Much like we stopped looking for fresh regions before, we
                 # can now stop looking for fresh zones; there are no more.
@@ -375,17 +408,17 @@ class Ring(object):
                         yield dev
                         used.add(dev_id)
                         same_zones.add(zone)
-                        ip_port = zone + (dev['ip'], dev['port'])
-                        same_ip_ports.add(ip_port)
+                        ip = zone + (dev['ip'],)
+                        same_ips.add(ip)
                         if len(same_zones) == self._num_zones:
                             hit_all_zones = True
                             break
 
-        hit_all_ip_ports = len(same_ip_ports) == self._num_ip_ports
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
-            if hit_all_ip_ports:
+        hit_all_ips = len(same_ips) == self._num_ips
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
+            if hit_all_ips:
                 # We've exhausted the pool of unused backends, so stop
                 # looking.
                 break
@@ -393,20 +426,19 @@ class Ring(object):
                 if handoff_part < len(part2dev_id):
                     dev_id = part2dev_id[handoff_part]
                     dev = self._devs[dev_id]
-                    ip_port = (dev['region'], dev['zone'],
-                               dev['ip'], dev['port'])
-                    if dev_id not in used and ip_port not in same_ip_ports:
+                    ip = (dev['region'], dev['zone'], dev['ip'])
+                    if dev_id not in used and ip not in same_ips:
                         yield dev
                         used.add(dev_id)
-                        same_ip_ports.add(ip_port)
-                        if len(same_ip_ports) == self._num_ip_ports:
-                            hit_all_ip_ports = True
+                        same_ips.add(ip)
+                        if len(same_ips) == self._num_ips:
+                            hit_all_ips = True
                             break
 
         hit_all_devs = len(used) == self._num_devs
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
             if hit_all_devs:
                 # We've used every device we have, so let's stop looking for
                 # unused devices now.
