@@ -15,20 +15,22 @@
 
 import os
 import itertools
+import json
 import time
 from collections import defaultdict
 from eventlet import Timeout
 
+from swift.container.sync_store import ContainerSyncStore
 from swift.container.backend import ContainerBroker, DATADIR
 from swift.container.reconciler import (
     MISPLACED_OBJECTS_ACCOUNT, incorrect_policy_index,
-    get_reconciler_container_name, get_row_to_q_entry_translater)
+    get_reconciler_container_name, get_row_to_q_entry_translator)
 from swift.common import db_replicator
 from swift.common.storage_policy import POLICIES
 from swift.common.exceptions import DeviceUnavailable
 from swift.common.http import is_success
 from swift.common.db import DatabaseAlreadyExists
-from swift.common.utils import (json, Timestamp, hash_path,
+from swift.common.utils import (Timestamp, hash_path,
                                 storage_directory, quorum_size)
 
 
@@ -59,7 +61,8 @@ class ContainerReplicator(db_replicator.Replicator):
                                 'storage_policy_index'))
         return sync_args
 
-    def _handle_sync_response(self, node, response, info, broker, http):
+    def _handle_sync_response(self, node, response, info, broker, http,
+                              different_region):
         parent = super(ContainerReplicator, self)
         if is_success(response.status):
             remote_info = json.loads(response.data)
@@ -74,7 +77,7 @@ class ContainerReplicator(db_replicator.Replicator):
                 broker.merge_timestamps(*(remote_info[key] for key in
                                           sync_timestamps))
         rv = parent._handle_sync_response(
-            node, response, info, broker, http)
+            node, response, info, broker, http, different_region)
         return rv
 
     def find_local_handoff_for_part(self, part):
@@ -166,14 +169,14 @@ class ContainerReplicator(db_replicator.Replicator):
         misplaced = broker.get_misplaced_since(point, self.per_diff)
         if not misplaced:
             return max_sync
-        translater = get_row_to_q_entry_translater(broker)
+        translator = get_row_to_q_entry_translator(broker)
         errors = False
         low_sync = point
         while misplaced:
             batches = defaultdict(list)
             for item in misplaced:
                 container = get_reconciler_container_name(item['created_at'])
-                batches[container].append(translater(item))
+                batches[container].append(translator(item))
             for container, item_list in batches.items():
                 success = self.feed_reconciler(container, item_list)
                 if not success:
@@ -187,6 +190,13 @@ class ContainerReplicator(db_replicator.Replicator):
     def _post_replicate_hook(self, broker, info, responses):
         if info['account'] == MISPLACED_OBJECTS_ACCOUNT:
             return
+
+        try:
+            self.sync_store.update_sync_store(broker)
+        except Exception:
+            self.logger.exception('Failed to update sync_store %s' %
+                                  broker.db_file)
+
         point = broker.get_reconciler_sync()
         if not broker.has_multiple_policies() and info['max_row'] != point:
             broker.update_reconciler_sync(info['max_row'])
@@ -208,6 +218,13 @@ class ContainerReplicator(db_replicator.Replicator):
             # this container shouldn't be here, make sure it's cleaned up
             self.reconciler_cleanups[broker.container] = broker
             return
+        try:
+            # DB is going to get deleted. Be preemptive about it
+            self.sync_store.remove_synced_container(broker)
+        except Exception:
+            self.logger.exception('Failed to remove sync_store entry %s' %
+                                  broker.db_file)
+
         return super(ContainerReplicator, self).delete_db(broker)
 
     def replicate_reconcilers(self):
@@ -235,6 +252,9 @@ class ContainerReplicator(db_replicator.Replicator):
     def run_once(self, *args, **kwargs):
         self.reconciler_containers = {}
         self.reconciler_cleanups = {}
+        self.sync_store = ContainerSyncStore(self.root,
+                                             self.logger,
+                                             self.mount_check)
         rv = super(ContainerReplicator, self).run_once(*args, **kwargs)
         if any([self.reconciler_containers, self.reconciler_cleanups]):
             self.replicate_reconcilers()
