@@ -15,11 +15,29 @@
 
 # This stuff can't live in test/unit/__init__.py due to its swob dependency.
 
+from collections import defaultdict
 from copy import deepcopy
 from hashlib import md5
 from swift.common import swob
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.utils import split_path
 
+from test.unit import FakeLogger, FakeRing
+
+
+class LeakTrackingIter(object):
+    def __init__(self, inner_iter, fake_swift, path):
+        self.inner_iter = inner_iter
+        self.fake_swift = fake_swift
+        self.path = path
+
+    def __iter__(self):
+        for x in self.inner_iter:
+            yield x
+
+    def close(self):
+        self.fake_swift.mark_closed(self.path)
+
 
 class FakeSwift(object):
     """
@@ -28,11 +46,27 @@ class FakeSwift(object):
 
     def __init__(self):
         self._calls = []
+        self._unclosed_req_paths = defaultdict(int)
         self.req_method_paths = []
         self.swift_sources = []
         self.uploaded = {}
         # mapping of (method, path) --> (response class, headers, body)
         self._responses = {}
+        self.logger = FakeLogger('fake-swift')
+        self.account_ring = FakeRing()
+        self.container_ring = FakeRing()
+        self.get_object_ring = lambda policy_index: FakeRing()
+
+    def _find_response(self, method, path):
+        resp = self._responses[(method, path)]
+        if isinstance(resp, list):
+            try:
+                resp = resp.pop(0)
+            except IndexError:
+                raise IndexError("Didn't find any more %r "
+                                 "in allowed responses" % (
+                                     (method, path),))
+        return resp
 
     def __call__(self, env, start_response):
         method = env['REQUEST_METHOD']
@@ -43,33 +77,35 @@ class FakeSwift(object):
             path += '?' + env['QUERY_STRING']
 
         if 'swift.authorize' in env:
-            resp = env['swift.authorize']()
+            resp = env['swift.authorize'](swob.Request(env))
             if resp:
                 return resp(env, start_response)
 
-        headers = swob.Request(env).headers
-        self._calls.append((method, path, headers))
+        req_headers = swob.Request(env).headers
         self.swift_sources.append(env.get('swift.source'))
 
         try:
-            resp_class, raw_headers, body = self._responses[(method, path)]
-            headers = swob.HeaderKeyDict(raw_headers)
+            resp_class, raw_headers, body = self._find_response(method, path)
+            headers = HeaderKeyDict(raw_headers)
         except KeyError:
             if (env.get('QUERY_STRING')
                     and (method, env['PATH_INFO']) in self._responses):
-                resp_class, raw_headers, body = self._responses[
-                    (method, env['PATH_INFO'])]
-                headers = swob.HeaderKeyDict(raw_headers)
+                resp_class, raw_headers, body = self._find_response(
+                    method, env['PATH_INFO'])
+                headers = HeaderKeyDict(raw_headers)
             elif method == 'HEAD' and ('GET', path) in self._responses:
-                resp_class, raw_headers, _ = self._responses[('GET', path)]
+                resp_class, raw_headers, body = self._find_response(
+                    'GET', path)
                 body = None
-                headers = swob.HeaderKeyDict(raw_headers)
+                headers = HeaderKeyDict(raw_headers)
             elif method == 'GET' and obj and path in self.uploaded:
                 resp_class = swob.HTTPOk
                 headers, body = self.uploaded[path]
             else:
-                print "Didn't find %r in allowed responses" % ((method, path),)
-                raise
+                raise KeyError("Didn't find %r in allowed responses" % (
+                    (method, path),))
+
+        self._calls.append((method, path, req_headers))
 
         # simulate object PUT
         if method == 'PUT' and obj:
@@ -87,12 +123,30 @@ class FakeSwift(object):
         req = swob.Request(env)
         resp = resp_class(req=req, headers=headers, body=body,
                           conditional_response=True)
-        return resp(env, start_response)
+        wsgi_iter = resp(env, start_response)
+        self.mark_opened(path)
+        return LeakTrackingIter(wsgi_iter, self, path)
+
+    def mark_opened(self, path):
+        self._unclosed_req_paths[path] += 1
+
+    def mark_closed(self, path):
+        self._unclosed_req_paths[path] -= 1
+
+    @property
+    def unclosed_requests(self):
+        return {path: count
+                for path, count in self._unclosed_req_paths.items()
+                if count > 0}
 
     @property
     def calls(self):
         return [(method, path) for method, path, headers in self._calls]
 
+    @property
+    def headers(self):
+        return [headers for method, path, headers in self._calls]
+
     @property
     def calls_with_headers(self):
         return self._calls
@@ -101,5 +155,8 @@ class FakeSwift(object):
     def call_count(self):
         return len(self._calls)
 
-    def register(self, method, path, response_class, headers, body):
+    def register(self, method, path, response_class, headers, body=''):
         self._responses[(method, path)] = (response_class, headers, body)
+
+    def register_responses(self, method, path, responses):
+        self._responses[(method, path)] = list(responses)
