@@ -13,7 +13,7 @@
 
 import unittest
 
-from swift.common.swob import Request
+from swift.common.swob import Request, wsgify, HTTPForbidden
 
 from swift.common.middleware import account_quotas
 
@@ -51,6 +51,10 @@ class FakeApp(object):
         self.headers = headers
 
     def __call__(self, env, start_response):
+        if 'swift.authorize' in env:
+            aresp = env['swift.authorize'](Request(env))
+            if aresp:
+                return aresp(env, start_response)
         if env['REQUEST_METHOD'] == "HEAD" and \
                 env['PATH_INFO'] == '/v1/a/c2/o2':
             env_key = get_object_env_key('a', 'c2', 'o2')
@@ -67,6 +71,21 @@ class FakeApp(object):
         return []
 
 
+class FakeAuthFilter(object):
+
+    def __init__(self, app):
+        self.app = app
+
+    @wsgify
+    def __call__(self, req):
+        def authorize(req):
+            if req.headers['x-auth-token'] == 'secret':
+                return
+            return HTTPForbidden(request=req)
+        req.environ['swift.authorize'] = authorize
+        return req.get_response(self.app)
+
+
 class TestAccountQuota(unittest.TestCase):
 
     def test_unauthorized(self):
@@ -77,8 +96,8 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        #Response code of 200 because authentication itself is not done here
-        self.assertEquals(res.status_int, 200)
+        # Response code of 200 because authentication itself is not done here
+        self.assertEqual(res.status_int, 200)
 
     def test_no_quotas(self):
         headers = [('x-account-bytes-used', '1000'), ]
@@ -88,7 +107,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_obj_request_ignores_attempt_to_set_quotas(self):
         # If you try to set X-Account-Meta-* on an object, it's ignored, so
@@ -102,7 +121,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_container_request_ignores_attempt_to_set_quotas(self):
         # As with an object, if you try to set X-Account-Meta-* on a
@@ -115,7 +134,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_bogus_quota_is_ignored(self):
         # This can happen if the metadata was set by a user prior to the
@@ -128,7 +147,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -139,7 +158,56 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_exceed_quota_not_authorized(self):
+        headers = [('x-account-bytes-used', '1000'),
+                   ('x-account-meta-quota-bytes', '0')]
+        app = FakeAuthFilter(
+            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o', method='PUT',
+                            headers={'x-auth-token': 'bad-secret'},
+                            environ={'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 403)
+
+    def test_exceed_quota_authorized(self):
+        headers = [('x-account-bytes-used', '1000'),
+                   ('x-account-meta-quota-bytes', '0')]
+        app = FakeAuthFilter(
+            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o', method='PUT',
+                            headers={'x-auth-token': 'secret'},
+                            environ={'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 413)
+
+    def test_under_quota_not_authorized(self):
+        headers = [('x-account-bytes-used', '0'),
+                   ('x-account-meta-quota-bytes', '1000')]
+        app = FakeAuthFilter(
+            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o', method='PUT',
+                            headers={'x-auth-token': 'bad-secret'},
+                            environ={'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 403)
+
+    def test_under_quota_authorized(self):
+        headers = [('x-account-bytes-used', '0'),
+                   ('x-account-meta-quota-bytes', '1000')]
+        app = FakeAuthFilter(
+            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o', method='PUT',
+                            headers={'x-auth-token': 'secret'},
+                            environ={'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 200)
 
     def test_over_quota_container_create_still_works(self):
         headers = [('x-account-bytes-used', '1001'),
@@ -151,7 +219,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_CONTAINER_META_BERT': 'ernie',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_over_quota_container_post_still_works(self):
         headers = [('x-account-bytes-used', '1001'),
@@ -163,7 +231,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_CONTAINER_META_BERT': 'ernie',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_over_quota_obj_post_still_works(self):
         headers = [('x-account-bytes-used', '1001'),
@@ -175,7 +243,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_OBJECT_META_BERT': 'ernie',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_copy_from(self):
         headers = [('x-account-bytes-used', '500'),
@@ -185,10 +253,11 @@ class TestAccountQuota(unittest.TestCase):
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
-                            'swift.cache': cache},
+                                     'swift.cache': cache},
                             headers={'x-copy-from': '/c2/o2'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
 
     def test_exceed_bytes_quota_copy_verb(self):
         headers = [('x-account-bytes-used', '500'),
@@ -198,10 +267,11 @@ class TestAccountQuota(unittest.TestCase):
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c2/o2',
                             environ={'REQUEST_METHOD': 'COPY',
-                            'swift.cache': cache},
+                                     'swift.cache': cache},
                             headers={'Destination': '/c/o'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
 
     def test_not_exceed_bytes_quota_copy_from(self):
         headers = [('x-account-bytes-used', '0'),
@@ -211,10 +281,10 @@ class TestAccountQuota(unittest.TestCase):
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
-                            'swift.cache': cache},
+                                     'swift.cache': cache},
                             headers={'x-copy-from': '/c2/o2'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_not_exceed_bytes_quota_copy_verb(self):
         headers = [('x-account-bytes-used', '0'),
@@ -224,10 +294,10 @@ class TestAccountQuota(unittest.TestCase):
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c2/o2',
                             environ={'REQUEST_METHOD': 'COPY',
-                            'swift.cache': cache},
+                                     'swift.cache': cache},
                             headers={'Destination': '/c/o'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_quota_copy_from_no_src(self):
         headers = [('x-account-bytes-used', '0'),
@@ -236,10 +306,10 @@ class TestAccountQuota(unittest.TestCase):
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
-                            'swift.cache': cache},
+                                     'swift.cache': cache},
                             headers={'x-copy-from': '/c2/o3'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_quota_copy_from_bad_src(self):
         headers = [('x-account-bytes-used', '0'),
@@ -248,10 +318,10 @@ class TestAccountQuota(unittest.TestCase):
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
-                            'swift.cache': cache},
+                                     'swift.cache': cache},
                             headers={'x-copy-from': 'bad_path'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 412)
+        self.assertEqual(res.status_int, 412)
 
     def test_exceed_bytes_quota_reseller(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -263,7 +333,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache,
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller_copy_from(self):
         headers = [('x-account-bytes-used', '500'),
@@ -277,7 +347,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'reseller_request': True},
                             headers={'x-copy-from': 'c2/o2'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller_copy_verb(self):
         headers = [('x-account-bytes-used', '500'),
@@ -291,7 +361,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'reseller_request': True},
                             headers={'Destination': 'c/o'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_bad_application_quota(self):
         headers = []
@@ -301,7 +371,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 404)
+        self.assertEqual(res.status_int, 404)
 
     def test_no_info_quota(self):
         headers = []
@@ -311,7 +381,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_not_exceed_bytes_quota(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -322,7 +392,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_invalid_quotas(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -334,7 +404,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': 'abc',
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 400)
+        self.assertEqual(res.status_int, 400)
 
     def test_valid_quotas_admin(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -345,7 +415,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache,
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '100'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_valid_quotas_reseller(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -357,7 +427,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '100',
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_delete_quotas(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -368,7 +438,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache,
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': ''})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_delete_quotas_with_remove_header(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -379,7 +449,7 @@ class TestAccountQuota(unittest.TestCase):
             'swift.cache': cache,
             'HTTP_X_REMOVE_ACCOUNT_META_QUOTA_BYTES': 'True'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_delete_quotas_reseller(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -389,7 +459,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '',
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_delete_quotas_with_remove_header_reseller(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -401,7 +471,7 @@ class TestAccountQuota(unittest.TestCase):
             'HTTP_X_REMOVE_ACCOUNT_META_QUOTA_BYTES': 'True',
             'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_invalid_request_exception(self):
         headers = [('x-account-bytes-used', '1000'), ]
@@ -412,7 +482,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache})
         res = req.get_response(app)
         # Response code of 200 because authentication itself is not done here
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
 
 if __name__ == '__main__':
