@@ -19,12 +19,15 @@ import shutil
 import tempfile
 import unittest
 import uuid
+import mock
 
 from swift.common import swob
 from swift.common.middleware import container_sync
 from swift.proxy.controllers.base import _get_cache_key
 from swift.proxy.controllers.info import InfoController
 
+from test.unit import FakeLogger
+
 
 class FakeApp(object):
 
@@ -39,7 +42,10 @@ class FakeApp(object):
             body = 'Response to Authorized Request'
         else:
             body = 'Pass-Through Response'
-        start_response('200 OK', [('Content-Length', str(len(body)))])
+        headers = [('Content-Length', str(len(body)))]
+        if 'HTTP_X_TIMESTAMP' in env:
+            headers.append(('X-Timestamp', env['HTTP_X_TIMESTAMP']))
+        start_response('200 OK', headers)
         return body
 
 
@@ -63,6 +69,94 @@ cluster_dfw1 = http://dfw1.host/v1/
     def tearDown(self):
         shutil.rmtree(self.tempdir, ignore_errors=1)
 
+    def test_current_not_set(self):
+        # no 'current' option set by default
+        self.assertIsNone(self.sync.realm)
+        self.assertIsNone(self.sync.cluster)
+        info = {}
+
+        def capture_swift_info(key, **options):
+            info[key] = options
+
+        with mock.patch(
+                'swift.common.middleware.container_sync.register_swift_info',
+                new=capture_swift_info):
+            self.sync.register_info()
+
+        for realm, realm_info in info['container_sync']['realms'].items():
+            for cluster, options in realm_info['clusters'].items():
+                self.assertEqual(options.get('current', False), False)
+
+    def test_current_invalid(self):
+        self.conf = {'swift_dir': self.tempdir, 'current': 'foo'}
+        self.sync = container_sync.ContainerSync(self.app, self.conf,
+                                                 logger=FakeLogger())
+        self.assertIsNone(self.sync.realm)
+        self.assertIsNone(self.sync.cluster)
+        info = {}
+
+        def capture_swift_info(key, **options):
+            info[key] = options
+
+        with mock.patch(
+                'swift.common.middleware.container_sync.register_swift_info',
+                new=capture_swift_info):
+            self.sync.register_info()
+
+        for realm, realm_info in info['container_sync']['realms'].items():
+            for cluster, options in realm_info['clusters'].items():
+                self.assertEqual(options.get('current', False), False)
+
+        error_lines = self.sync.logger.get_lines_for_level('error')
+        self.assertEqual(error_lines, ['Invalid current '
+                                       '//REALM/CLUSTER (foo)'])
+
+    def test_current_in_realms_conf(self):
+        self.conf = {'swift_dir': self.tempdir, 'current': '//us/dfw1'}
+        self.sync = container_sync.ContainerSync(self.app, self.conf)
+        self.assertEqual('US', self.sync.realm)
+        self.assertEqual('DFW1', self.sync.cluster)
+        info = {}
+
+        def capture_swift_info(key, **options):
+            info[key] = options
+
+        with mock.patch(
+                'swift.common.middleware.container_sync.register_swift_info',
+                new=capture_swift_info):
+            self.sync.register_info()
+
+        for realm, realm_info in info['container_sync']['realms'].items():
+            for cluster, options in realm_info['clusters'].items():
+                if options.get('current'):
+                    break
+        self.assertEqual(realm, self.sync.realm)
+        self.assertEqual(cluster, self.sync.cluster)
+
+    def test_missing_from_realms_conf(self):
+        self.conf = {'swift_dir': self.tempdir, 'current': 'foo/bar'}
+        self.sync = container_sync.ContainerSync(self.app, self.conf,
+                                                 logger=FakeLogger())
+        self.assertEqual('FOO', self.sync.realm)
+        self.assertEqual('BAR', self.sync.cluster)
+        info = {}
+
+        def capture_swift_info(key, **options):
+            info[key] = options
+
+        with mock.patch(
+                'swift.common.middleware.container_sync.register_swift_info',
+                new=capture_swift_info):
+            self.sync.register_info()
+
+        for realm, realm_info in info['container_sync']['realms'].items():
+            for cluster, options in realm_info['clusters'].items():
+                self.assertEqual(options.get('current', False), False)
+
+        for line in self.sync.logger.get_lines_for_level('error'):
+            self.assertEqual(line, 'Unknown current '
+                             '//REALM/CLUSTER (//FOO/BAR)')
+
     def test_pass_through(self):
         req = swob.Request.blank('/v1/a/c')
         resp = req.get_response(self.sync)
@@ -123,18 +217,20 @@ cluster_dfw1 = http://dfw1.host/v1/
             req.environ.get('swift.log_info'))
 
     def test_valid_sig(self):
+        ts = '1455221706.726999_0123456789abcdef'
         sig = self.sync.realms_conf.get_sig(
-            'GET', '/v1/a/c', '0', 'nonce',
+            'GET', '/v1/a/c', ts, 'nonce',
             self.sync.realms_conf.key('US'), 'abc')
-        req = swob.Request.blank(
-            '/v1/a/c', headers={'x-container-sync-auth': 'US nonce ' + sig})
+        req = swob.Request.blank('/v1/a/c', headers={
+            'x-container-sync-auth': 'US nonce ' + sig,
+            'x-backend-inbound-x-timestamp': ts})
         req.environ[_get_cache_key('a', 'c')[1]] = {'sync_key': 'abc'}
         resp = req.get_response(self.sync)
         self.assertEqual(resp.status, '200 OK')
         self.assertEqual(resp.body, 'Response to Authorized Request')
-        self.assertTrue(
-            'cs:valid' in req.environ.get('swift.log_info'),
-            req.environ.get('swift.log_info'))
+        self.assertIn('cs:valid', req.environ.get('swift.log_info'))
+        self.assertIn('X-Timestamp', resp.headers)
+        self.assertEqual(ts, resp.headers['X-Timestamp'])
 
     def test_valid_sig2(self):
         sig = self.sync.realms_conf.get_sig(
