@@ -57,57 +57,93 @@ class TestCatchErrors(unittest.TestCase):
         app = catch_errors.CatchErrorMiddleware(FakeApp(), {})
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
-        self.assertEquals(list(resp), ['FAKE APP'])
+        self.assertEqual(list(resp), ['FAKE APP'])
 
     def test_catcherrors(self):
         app = catch_errors.CatchErrorMiddleware(FakeApp(True), {})
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
-        self.assertEquals(list(resp), ['An error occurred'])
+        self.assertEqual(list(resp), ['An error occurred'])
 
     def test_trans_id_header_pass(self):
-        self.assertEquals(self.logger.txn_id, None)
+        self.assertEqual(self.logger.txn_id, None)
 
         def start_response(status, headers, exc_info=None):
-            self.assert_('X-Trans-Id' in (x[0] for x in headers))
+            self.assertTrue('X-Trans-Id' in (x[0] for x in headers))
         app = catch_errors.CatchErrorMiddleware(FakeApp(), {})
         req = Request.blank('/v1/a/c/o')
         app(req.environ, start_response)
-        self.assertEquals(len(self.logger.txn_id), 34)  # 32 hex + 'tx'
+        self.assertEqual(len(self.logger.txn_id), 34)  # 32 hex + 'tx'
 
     def test_trans_id_header_fail(self):
-        self.assertEquals(self.logger.txn_id, None)
+        self.assertEqual(self.logger.txn_id, None)
 
         def start_response(status, headers, exc_info=None):
-            self.assert_('X-Trans-Id' in (x[0] for x in headers))
+            self.assertTrue('X-Trans-Id' in (x[0] for x in headers))
         app = catch_errors.CatchErrorMiddleware(FakeApp(True), {})
         req = Request.blank('/v1/a/c/o')
         app(req.environ, start_response)
-        self.assertEquals(len(self.logger.txn_id), 34)
+        self.assertEqual(len(self.logger.txn_id), 34)
 
     def test_error_in_iterator(self):
         app = catch_errors.CatchErrorMiddleware(
             FakeApp(body_iter=(int(x) for x in 'abcd')), {})
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
-        self.assertEquals(list(resp), ['An error occurred'])
+        self.assertEqual(list(resp), ['An error occurred'])
 
     def test_trans_id_header_suffix(self):
-        self.assertEquals(self.logger.txn_id, None)
+        self.assertEqual(self.logger.txn_id, None)
 
         def start_response(status, headers, exc_info=None):
-            self.assert_('X-Trans-Id' in (x[0] for x in headers))
+            self.assertTrue('X-Trans-Id' in (x[0] for x in headers))
         app = catch_errors.CatchErrorMiddleware(
             FakeApp(), {'trans_id_suffix': '-stuff'})
         req = Request.blank('/v1/a/c/o')
         app(req.environ, start_response)
         self.assertTrue(self.logger.txn_id.endswith('-stuff'))
 
+    def test_trans_id_header_extra(self):
+        self.assertEqual(self.logger.txn_id, None)
+
+        def start_response(status, headers, exc_info=None):
+            self.assertTrue('X-Trans-Id' in (x[0] for x in headers))
+        app = catch_errors.CatchErrorMiddleware(
+            FakeApp(), {'trans_id_suffix': '-fromconf'})
+        req = Request.blank('/v1/a/c/o',
+                            headers={'X-Trans-Id-Extra': 'fromuser'})
+        app(req.environ, start_response)
+        self.assertTrue(self.logger.txn_id.endswith('-fromconf-fromuser'))
+
+    def test_trans_id_header_extra_length_limit(self):
+        self.assertEqual(self.logger.txn_id, None)
+
+        def start_response(status, headers, exc_info=None):
+            self.assertTrue('X-Trans-Id' in (x[0] for x in headers))
+        app = catch_errors.CatchErrorMiddleware(
+            FakeApp(), {'trans_id_suffix': '-fromconf'})
+        req = Request.blank('/v1/a/c/o',
+                            headers={'X-Trans-Id-Extra': 'a' * 1000})
+        app(req.environ, start_response)
+        self.assertTrue(self.logger.txn_id.endswith(
+            '-aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'))
+
+    def test_trans_id_header_extra_quoted(self):
+        self.assertEqual(self.logger.txn_id, None)
+
+        def start_response(status, headers, exc_info=None):
+            self.assertTrue('X-Trans-Id' in (x[0] for x in headers))
+        app = catch_errors.CatchErrorMiddleware(FakeApp(), {})
+        req = Request.blank('/v1/a/c/o',
+                            headers={'X-Trans-Id-Extra': 'xan than"gum'})
+        app(req.environ, start_response)
+        self.assertTrue(self.logger.txn_id.endswith('-xan%20than%22gum'))
+
     def test_catcherrors_with_unexpected_error(self):
         app = catch_errors.CatchErrorMiddleware(FakeApp(error='strange'), {})
         req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
         resp = app(req.environ, start_response)
-        self.assertEquals(list(resp), ['An error occurred'])
+        self.assertEqual(list(resp), ['An error occurred'])
 
 
 if __name__ == '__main__':
